// Vertex Shader

#version 330 core

layout(location = 0) in vec3 in_position;
layout(location = 1) in vec3 in_normal;
layout(location = 2) in vec2 in_tex;
//layout(location = 3) in vec3 in_tangent;
//layout(location = 4) in vec3 in_bitangent;

uniform mat4 M;//model matrix
uniform mat4 V;//view matrix
uniform mat4 P;//projection matrix

//Direction of the light in world space
uniform vec3 lightdir_world;

out vec3 N;
out vec3 L;
out vec3 E;

out vec2 tex;

//Contour-Stuff
out vec4 edgeinfo;

//Shadow-Stuff
uniform mat4 sm_light_texmat;
out vec4 SM_tex_coord;

mat4 M_t;
mat4 M_s;

void main(void)
{

		
	//Direction to light in cameraspace
	vec3 lightdir_cam = (V * vec4(lightdir_world, 0)).xyz;
	
	//Vertexnormal in cameraspace
	vec3 normal_cam = (V * M * vec4(in_normal ,0)).xyz;// Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.

	//Position of the vertex in cameraspace
	vec3 position_cam = (V * M * vec4(in_position,1)).xyz;
	//Direction to camera from vertex, camera in cameraspace is at 0,0,0
	vec3 eyedir_cam = vec3(0,0,0) - position_cam;

	//ES_z.a = -(position_cam.z-0.1)/(100.0-0.1); // 100 is the far plane dist => mat4 Projection = perspective(45.0f, 4.0f / 3.0f, 0.1f, 100.0f);


	N = normalize(normal_cam);
	L = normalize(lightdir_cam);
	E = normalize(eyedir_cam);
	
	
	edgeinfo = vec4(max(dot(N,E),0),0,0,1); //important alpha = 1

	//smtexcoord = Mt * Ms * Plight * Vlight * M * v == texmat * M * v

	SM_tex_coord = sm_light_texmat * M * vec4(in_position, 1);
	
	//Forward UV
	tex = in_tex;

	//Apply MVP
	gl_Position =  P * V * M * vec4(in_position, 1);
}
